/*
 * Vifir - An electronic flight bag wannabe utility for aircraft pilots
 * Copyright (C) 2009 David Ludlow
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * dave@adsllc.com, PO Box 13070, Toledo, OH 43623
 */

#ifndef CHARTCHOOSERDRAWINGAREA_H_
#define CHARTCHOOSERDRAWINGAREA_H_

#include <vector>
#include <string>
#include <libglademm.h>
#include <gtkmm/drawingarea.h>
#include <gdkmm/pixbuf.h>
#include <proj_api.h>
#include "DataStore.h"
#include "GeoChart.h"
#include "Route.h"

class ChartChooserDrawingArea : public Gtk::DrawingArea
{
public:
	enum Area_enum {
		AREA_CONTIGUOUS,
		AREA_ALASKA,
		AREA_HAWAII
	};

	ChartChooserDrawingArea(BaseObjectType* cobject,const Glib::RefPtr<Gnome::Glade::Xml>& glade_xml);
	virtual ~ChartChooserDrawingArea();

	void Init(
		std::string BackgroundImgPath,
		DataStore *Data,
		enum Area_enum Area,
		std::vector<GeoChart*>::iterator *SelectedCharti,
		Route *RouteData
		);
	void DrawGeoChartBounds(Cairo::RefPtr<Cairo::Context> Context,GeoChart *Chart);
	void LonLatToPixel(double Lon, double Lat, double *X, double *Y);
	void SetScaleOfs(double Lon1, double Lat1, double X1, double Y1,
					double Lon2, double Lat2, double X2, double Y2);
	void SetChartType(GeoChart::Type_enum ChartType);
	void Redraw();
	void DrawGreatCircleRoute(Route *RouteData);
	std::vector<GeoChart*>::iterator GetChartByLoc(double X, double Y);

private:
	bool on_expose_event(GdkEventExpose* ev);
	bool on_button_release_event(GdkEventButton * event);
	bool on_motion_notify_event(GdkEventMotion* event);

	std::string BackgroundImgPath;
	Glib::RefPtr<Gdk::Pixbuf> BgImgPixbuf;
	DataStore *Data;
	projPJ Proj;
	double XScale,XOfs,YScale,YOfs;
	enum GeoChart::Type_enum ChartType;
	std::vector<GeoChart*>::iterator *SelectedCharti, HoveredCharti; 
	Route *RouteData;
};

#endif /*CHARTCHOOSERDRAWINGAREA_H_*/
