/*
 * Vifir - An electronic flight bag wannabe utility for aircraft pilots
 * Copyright (C) 2009 David Ludlow
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * dave@adsllc.com, PO Box 13070, Toledo, OH 43623
 */

#ifndef FREQUENCY_H_
#define FREQUENCY_H_

#include <stdint.h>
#include <string>

class Frequency
{
public:
	enum TypeEnum {
		MISC,
		AWOS_A,
		AWOS_1,
		AWOS_2,
		AWOS_3,
		ASOS,
		ATIS,
		CLEARANCEDELIVERY,
		GROUND,
		TOWER,
		DEPARTURE,
		APPROACH,
		CENTER,
		FLIGHTSERVICE,
		UNICOM,
		CTAF,
		VOR,
		NDB,
		LOCALIZER	
	};
	
	enum UnitsEnum {
		HZ,
		KHZ,
		MHZ,
		GHZ
	};

	static const std::string UnitsText[4];

	Frequency(unsigned int RawFreq=0, unsigned int Precision=3, UnitsEnum Units=MHZ, TypeEnum Type=MISC);
	double ToDouble();
	std::string FreqToString();	/* Does  not include units */
	std::string ToString();		/* Includes units */

	/* Accessors */
	unsigned int GetRawFreq();
	void SetRawFreq(unsigned int RawFreq);
	unsigned char GetPrecision();
	void SetPrecision(unsigned char Precision);
	enum UnitsEnum GetUnits();
	void SetUnits(UnitsEnum Units);
	enum TypeEnum GetType();
	void SetType(TypeEnum Type);
	std::string GetDesc();
	void SetDesc(std::string Desc);
	std::string GetAirportICAO();

	bool ParseNFD(std::string Line);
	bool ParseXPlane850(std::string *Line, std::string AirportICAO);
	void Clone(Frequency *SrcFreq);
	bool MakeBinary(struct Frequency_Binary_struct *BinaryData);
	void SetFromBinary(struct Frequency_Binary_struct *BinaryData);

private:
	unsigned int RawFreq;		/* Frequency digits */
	unsigned char Precision;	/* Location of the decimal point */
	enum UnitsEnum Units;
	enum TypeEnum Type;
	std::string Desc;
	std::string AirportICAO;
};

/* Increment the version when the structure changes */
#define FREQUENCY_BINARY_STRUCT_VER 1
#ifdef WIN32
#pragma pack(1)
#endif
struct Frequency_Binary_struct {
	uint32_t RawFreq;
	uint8_t Precision;
	uint8_t Units;
	uint8_t Type;
	char Desc[255];
	/* AirportICAO is not saved since it's only used while parsing the NFD */
}
#ifndef WIN32
        __attribute__ ((packed))
#endif
;
#ifdef WIN32
#pragma pack()
#endif

#endif /*FREQUENCY_H_*/
