/*
 * Vifir - An electronic flight bag wannabe utility for aircraft pilots
 * Copyright (C) 2009 David Ludlow
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * dave@adsllc.com, PO Box 13070, Toledo, OH 43623
 */

#define __STDC_LIMIT_MACROS
#include <stdint.h>
#include <cstring>

#include "Waypoint.h"
#include "Util.h"

Waypoint::Waypoint()
{
	Lon=NAN;
	Lat=NAN;
	LocType=LOC_UNKNOWN;
	ID="";
	Name="";
}

std::string Waypoint::GetID()
{
	return ID;
}

double Waypoint::GetLon()
{
	return Lon;
}

double Waypoint::GetLat()
{
	return Lat;
}

void Waypoint::SetLonLat(double Lon, double Lat)
{
	this->Lon=Lon;
	this->Lat=Lat;
}

bool Waypoint::Matchi(std::string Str)
{
	Str=Util::toUpper(Str);
	if (ID.find(Str)!=Str.npos)
		return true;

	if (Name.find(Str)!=Str.npos)
		return true;

	return false;
}

enum Waypoint::LocType_enum Waypoint::GetLocType()
{
	return LocType;
}

std::string Waypoint::GetLocTypeStr(enum LocType_enum LocType)
{
	const char LocType_str[][20]={"Unknown","Airport","NavAid","EnRoute Waypoint"};

	if (LocType<=3)
		return LocType_str[LocType];

	return "Unknown";
}

/* Return our location type string if none explicitly specified */
std::string Waypoint::GetLocTypeStr()
{
	return GetLocTypeStr(this->LocType);
}

std::string Waypoint::GetName()
{
	return Name;
}

#ifdef HAVE_LIBNOVA
time_t Waypoint::GetMornCivilTwilight() {
	struct ln_lnlat_posn pos;
	struct ln_rst_time rst;
	time_t Time;

	pos.lat=Lat;
	pos.lng=Lon;
	if (ln_get_solar_rst_horizon (ln_get_julian_from_sys(), &pos, LN_SOLAR_CIVIL_HORIZON, &rst)==0) {
		ln_get_timet_from_julian(rst.rise,&Time);
		return Time;
	} else {
		return 0;
	}
}

time_t Waypoint::GetSunrise() {
	struct ln_lnlat_posn pos;
	struct ln_rst_time rst;
	time_t Time;

	pos.lat=Lat;
	pos.lng=Lon;
	if (ln_get_solar_rst_horizon (ln_get_julian_from_sys(), &pos, LN_SOLAR_STANDART_HORIZON, &rst)==0) {
		ln_get_timet_from_julian(rst.rise,&Time);
		return Time;
	} else {
		return 0;
	}
}

time_t Waypoint::GetSunset() {
	struct ln_lnlat_posn pos;
	struct ln_rst_time rst;
	time_t Time;

	pos.lat=Lat;
	pos.lng=Lon;
	if (ln_get_solar_rst_horizon (ln_get_julian_from_sys(), &pos, LN_SOLAR_STANDART_HORIZON, &rst)==0) {
		ln_get_timet_from_julian(rst.set,&Time);
		return Time;
	} else {
		return 0;
	}
}

time_t Waypoint::GetEveCivilTwilight() {
	struct ln_lnlat_posn pos;
	struct ln_rst_time rst;
	time_t Time;

	pos.lat=Lat;
	pos.lng=Lon;
	if (ln_get_solar_rst_horizon (ln_get_julian_from_sys(), &pos, LN_SOLAR_CIVIL_HORIZON, &rst)==0) {
		ln_get_timet_from_julian(rst.set,&Time);
		return Time;
	} else {
		return 0;
	}
}
#endif /* HAVE_LIBNOVA */

bool Waypoint::MakeBinary(struct Waypoint_Binary_struct *BinaryData) {
	double Lonx1000000,Latx1000000;

	Lonx1000000=Lon*1000000.0;
	Latx1000000=Lat*1000000.0;
	/* Sanity checking */
	if (
		ID.size()>sizeof(BinaryData->ID) ||
		Name.size()>sizeof(BinaryData->Name) ||
		Lonx1000000<INT32_MIN || Lonx1000000>INT32_MAX ||
		Latx1000000<INT32_MIN || Latx1000000>INT32_MAX ||
		LocType>UINT8_MAX
		) {
		return false;
	}

	strncpy(BinaryData->ID,ID.c_str(),sizeof(BinaryData->ID));
	strncpy(BinaryData->Name,Name.c_str(),sizeof(BinaryData->Name));
	BinaryData->Lonx1000000=(int32_t)round(Lonx1000000);
	BinaryData->Latx1000000=(int32_t)round(Latx1000000);
	BinaryData->LocType=LocType;

	return true;
}

void Waypoint::SetFromBinary(struct Waypoint_Binary_struct *BinaryData) {
	ID.assign(BinaryData->ID,sizeof(BinaryData->ID));
	Name.assign(BinaryData->Name,sizeof(BinaryData->Name));
	Lon=BinaryData->Lonx1000000/1000000.0;
	Lat=BinaryData->Latx1000000/1000000.0;
	LocType=(enum LocType_enum)BinaryData->LocType;

	/* Reset the size to the actual number of characters */
	ID=ID.c_str();
	Name=Name.c_str();
}
